<?php
/**
 * Custom functions that act independently of the theme templates
 *
 * Eventually, some of the functionality here could be replaced by core features
 *
 * @package Infinity
 */

/**
 * Adds custom classes to the array of body classes.
 * ================================================
 *
 * @param array $classes Classes for the body element.
 *
 * @return array
 */
function infinity_body_classes( $classes ) {
	// Adds a class of group-blog to blogs with more than 1 published author.
	if ( is_multi_author() ) {
		$classes[] = 'group-blog';
	}

	global $infinity_custom_class;
	if ( $infinity_custom_class ) {
		$classes[] = $infinity_custom_class;
	}

	if ( Kirki::get_option( 'tm-builder', 'site_general_boxed' ) == 1 ) {
		$classes[] = 'boxed';
	}

	$classes[] = Kirki::get_option( 'tm-builder', 'header_type' );

	global $infinity_page_layout_private;
	if ( $infinity_page_layout_private != 'default' && class_exists( 'cmb2_bootstrap_205' ) ) {
		$layout = get_post_meta( get_the_ID(), "infinity_page_layout_private", true );
	} else {
		$layout = Kirki::get_option( 'tm-builder', 'page_layout' );
	}

	if ( is_singular( 'project' ) ) {
		$classes[] = 'project-template-' . get_post_meta( get_the_ID(), "_template", true );
	}

	if ( is_single() && get_post_meta( get_the_ID(), "infinity_use_vc", true ) ) {
		$classes[] = 'use-vc';
	}

	if ( get_post_meta( get_the_ID(), "infinity_post_layout_private", true ) ) {
		$classes[] = get_post_meta( get_the_ID(), "infinity_post_layout_private", true );
	}

	$classes[] = $layout;

	return $classes;
}

add_filter( 'body_class', 'infinity_body_classes' );

if ( version_compare( $GLOBALS['wp_version'], '4.1', '<' ) ) : /**
 * Filters wp_title to print a neat <title> tag based on what is being viewed.
 * ==========================================================================
 *
 * @param string $title Default title text for current view.
 * @param string $sep Optional separator.
 *
 * @return string The filtered title.
 */ {
	function infinity_wp_title( $title, $sep ) {
		if ( is_feed() ) {
			return $title;
		}

		global $page, $paged;

		// Add the blog name.
		$title .= get_bloginfo( 'name', 'display' );

		// Add the blog description for the home/front page.
		$site_description = get_bloginfo( 'description', 'display' );
		if ( $site_description && ( is_home() || is_front_page() ) ) {
			$title .= " $sep $site_description";
		}

		// Add a page number if necessary.
		if ( ( $paged >= 2 || $page >= 2 ) && ! is_404() ) {
			$title .= " $sep " . sprintf( esc_html__( 'Page %s', 'tm-builder' ), max( $paged, $page ) );
		}

		return $title;
	}

	add_filter( 'wp_title', 'infinity_wp_title', 10, 2 );

	/**
	 * Title shim for sites older than WordPress 4.1.
	 *
	 * @link https://make.wordpress.org/core/2014/10/29/title-tags-in-4-1/
	 * @todo Remove this function when WordPress 4.3 is released.
	 */
	function infinity_render_title() {
		?>
		<title><?php wp_title( '|', true, 'right' ); ?></title>
		<?php
	}

	add_action( 'wp_head', 'infinity_render_title' );
}
endif;

/***
 * Get mini cart HTML
 * ==================
 * @return string
 */
if ( class_exists( 'WooCommerce' ) ) {
	function thememove__minicart() {
		$cart_html = '';
		$qty       = WC()->cart->get_cart_contents_count();
		$cart_html .= '<div class="mini-cart__button" title="' . esc_html__( 'View your shopping cart', 'tm-builder' ) . '">';
		$cart_html .= '<span class="mini-cart-icon"' . ' data-count="' . esc_attr( $qty ) . '"></span>';
		$cart_html .= '</div>';

		return $cart_html;
	}

	add_filter( 'woocommerce_add_to_cart_fragments', 'thememove_header_add_to_cart_fragment' );
}

/**
 * Ensure cart contents update when products are added to the cart via AJAX
 * ========================================================================
 *
 * @param $fragments
 *
 * @return mixed
 */
if ( class_exists( 'WooCommerce' ) ) {
	function thememove_header_add_to_cart_fragment( $fragments ) {
		ob_start();
		echo thememove__minicart();
		$fragments['.mini-cart__button'] = ob_get_clean();

		return $fragments;
	}
}

/**
 * Custom Comment Form
 * ========================================================================
 */
function infinity_comment( $comment, $args, $depth ) {
	$GLOBALS['comment'] = $comment; ?>
	<li <?php comment_class(); ?> id="li-comment-<?php comment_ID() ?>">
	<div id="comment-<?php comment_ID(); ?>">
		<div class="comment-author vcard">
			<?php echo get_avatar( $comment, $size = '100' ); ?>
		</div>
		<div class="comment-content">
			<?php if ( $comment->comment_approved == '0' ) : ?>
				<em><?php _e( 'Your comment is awaiting moderation.', 'tm-builder' ) ?></em>
				<br/>
			<?php endif; ?>
			<div class="metadata">
				<?php printf( esc_html__( '<cite class="fn">%s</cite>', 'tm-builder' ), get_comment_author_link() ) ?>
				<br/>
				<a href="<?php echo htmlspecialchars( get_comment_link( $comment->comment_ID ) ) ?>">
					<?php printf( esc_html__( '%1$s', 'tm-builder' ), get_comment_date(), get_comment_time() ) ?></a>
				<?php edit_comment_link( esc_html__( '(Edit)', 'tm-builder' ), '  ', '' ) ?>
			</div>
			<?php comment_text() ?>
			<?php comment_reply_link( array_merge( $args, array(
				'depth'     => $depth,
				'max_depth' => $args['max_depth']
			) ) ) ?>
		</div>
	</div>
	<?php
}

function custom_excerpt_length( $length ) {
	return 20;
}

add_filter( 'excerpt_length', 'custom_excerpt_length', 999 );

function infinity_projects_fields( $fields ) {
	$fields['template'] = array(
		'name'        => esc_html__( 'Template', 'tm-builder' ),
		'description' => esc_html__( 'Choose template for this project.', 'tm-builder' ),
		'type'        => 'select',
		'default'     => '1',
		'section'     => 'info',
		'options'     => array(
			'1' => 'Template 1',
			'2' => 'Template 2',
			'3' => 'Template 3',
			'9' => 'Custom Template'
		)
	);

	$fields['location'] = array(
		'name'        => esc_html__( 'Location', 'tm-builder' ),
		'description' => esc_html__( 'Enter a location for this project.', 'tm-builder' ),
		'type'        => 'text',
		'default'     => '',
		'section'     => 'info'
	);

	$fields['surface_area'] = array(
		'name'        => esc_html__( 'Surface Area', 'tm-builder' ),
		'description' => esc_html__( 'Enter a surface area for this project.', 'tm-builder' ),
		'type'        => 'text',
		'default'     => '',
		'section'     => 'info'
	);

	$fields['year_completed'] = array(
		'name'        => esc_html__( 'Year Completed', 'tm-builder' ),
		'description' => esc_html__( 'Enter a year Completed for this project.', 'tm-builder' ),
		'type'        => 'text',
		'default'     => '',
		'section'     => 'info'
	);

	$fields['value'] = array(
		'name'        => esc_html__( 'Value', 'tm-builder' ),
		'description' => esc_html__( 'Enter a value for this project.', 'tm-builder' ),
		'type'        => 'text',
		'default'     => '',
		'section'     => 'info'
	);

	$fields['architect'] = array(
		'name'        => esc_html__( 'Architect', 'tm-builder' ),
		'description' => esc_html__( 'Enter a architect for this project.', 'tm-builder' ),
		'type'        => 'text',
		'default'     => '',
		'section'     => 'info'
	);

	return $fields;
}

add_filter( 'projects_custom_fields', 'infinity_projects_fields' );

class ThemeMove_RecentPosts extends WP_Widget {

	function ThemeMove_RecentPosts() {
		parent::__construct( false, 'Theme Move Recent Posts' );
	}

	function widget( $args, $instance ) {
		$title = apply_filters( 'widget_title', $instance['title'] );
		$cat   = $instance['cat'];
		$num   = $instance['num'];
		echo wp_kses( $args['before_widget'], array(
			'aside' => array( 'id' => array(), 'class' => array() ),
			'div'   => array( 'id' => array(), 'class' => array() )
		) );
		if ( $cat == 'c1' ) {
			if ( ! empty( $title ) ) {
				echo wp_kses( $args['before_title'] . $title . $args['after_title'], array(
					'h3' => array( 'class' => array() )
				) );
			} else {
				echo wp_kses( $args['before_title'] . '&nbsp;' . $args['after_title'], array(
					'h3' => array( 'class' => array() )
				) );
			}
			$tmrp_args = array(
				'post_type'           => 'post',
				'ignore_sticky_posts' => 1,
				'posts_per_page'      => $num
			);
		} elseif ( $cat == 'c3' ) {
			if ( ! empty( $title ) ) {
				echo wp_kses( $args['before_title'] . $title . $args['after_title'], array(
					'h3' => array( 'class' => array() )
				) );
			} else {
				echo wp_kses( $args['before_title'] . '&nbsp;' . $args['after_title'], array(
					'h3' => array( 'class' => array() )
				) );
			}
			$sticky    = get_option( 'sticky_posts' );
			$tmrp_args = array(
				'post_type'      => 'post',
				'post__in'       => $sticky,
				'posts_per_page' => $num
			);
		} else {
			echo wp_kses( $args['before_title'] . '<a href="' . esc_url( get_catery_link( $cat ) ) . '" title="' . esc_attr( get_cat_name( $cat ) ) . '">' . get_cat_name( $cat ) . '</a>' . $args['after_title'], array(
				'h3' => array( 'class' => array() ),
				'a'  => array( 'href' => array(), 'title' => array() )
			) );
			$tmrp_args = array(
				'post_type'           => 'post',
				'cat'                 => $cat,
				'ignore_sticky_posts' => 1,
				'posts_per_page'      => $num
			);
		}
		$tmrp_query = new WP_Query( $tmrp_args );
		if ( $tmrp_query->have_posts() ) {
			while ( $tmrp_query->have_posts() ) {
				$tmrp_query->the_post();
				?>
				<div class="thememove-recent-posts-list">
					<div class="thememove-recent-posts-thumb">
						<?php if ( has_post_thumbnail() ) {
							the_post_thumbnail( array( 80, 53 ) );
						} ?>
					</div>
					<div class="thememove-recent-posts-info">
						<a href="<?php the_permalink(); ?>"
						   title="<?php the_title(); ?>"><?php the_title(); ?>
						</a>
						<?php the_time( 'F j, Y' ); ?>
					</div>
				</div>
				<?php
			}
		}
		wp_reset_postdata();
		echo wp_kses( $args['after_widget'], array(
			'aside' => array()
		) );
	}

	function update( $new_instance, $old_instance ) {
		$instance          = array();
		$instance['title'] = ( ! empty( $new_instance['title'] ) ) ? strip_tags( $new_instance['title'] ) : '';
		$instance['cat']   = ( ! empty( $new_instance['cat'] ) ) ? strip_tags( $new_instance['cat'] ) : '';
		$instance['num']   = ( ! empty( $new_instance['num'] ) ) ? strip_tags( $new_instance['num'] ) : '';

		return $instance;
	}

	function form( $instance ) {
		if ( isset( $instance['title'] ) ) {
			$title = $instance['title'];
		} else {
			$title = esc_html__( 'New title', 'tm-builder' );
		}
		$cat = $instance['cat'];
		$num = $instance['num'];
		?>
		<p>
			<label
				for="<?php echo esc_attr( $this->get_field_id( 'title' ) ); ?>"><?php _e( 'Title:', 'tm-builder' ); ?></label>
			<input class="widefat" id="<?php echo esc_attr( $this->get_field_id( 'title' ) ); ?>"
			       name="<?php echo esc_attr( $this->get_field_name( 'title' ) ); ?>" type="text"
			       value="<?php echo esc_attr( $title ); ?>"/>
		</p>
		<p>
			<label
				for="<?php echo esc_attr( $this->get_field_id( 'cat' ) ); ?>"><?php _e( 'Category:', 'tm-builder' ); ?></label>
			<select name="<?php echo esc_attr( $this->get_field_name( 'cat' ) ); ?>">
				<option value="c1" <?php
				if ( $cat == 'c1' ) {
					echo 'selected';
				}
				?>>Recent
				</option>
				<option value="c3" <?php
				if ( $cat == 'c3' ) {
					echo 'selected';
				}
				?>>Sticky
				</option>
				<?php
				$categories = get_categories( 'hide_empty=0' );
				if ( $categories ) {
					foreach ( $categories as $category ) {
						$sl = '';
						if ( $category->term_id == $cat ) {
							$sl = 'selected';
						}
						echo '<option value="' . esc_attr( $category->term_id ) . '" ' . $sl . '>' . $category->name . '</option>';
					}
				}
				?>
			</select>
		</p>
		<p>
			<label
				for="<?php echo esc_attr( $this->get_field_id( 'num' ) ); ?>"><?php _e( 'Number:', 'tm-builder' ); ?></label>
			<select name="<?php echo esc_attr( $this->get_field_name( 'num' ) ); ?>">
				<?php
				for ( $i = 1; $i <= 40; $i ++ ) {
					$sl = '';
					if ( $i == $num ) {
						$sl = 'selected';
					}
					echo '<option value="' . esc_attr( $i ) . '" ' . $sl . '>' . $i . '</option>';
				}
				?>
			</select>
		</p>
		<?php
	}
}

function thememove_widgets_init() {
	register_widget( 'ThemeMove_RecentPosts' );
}

add_action( 'widgets_init', 'thememove_widgets_init' );

// Remove admin notification of Projects
if ( class_exists( 'tm-builder' ) ) {
	global $projects;
	remove_action( 'admin_notices', array( $projects->admin, 'configuration_admin_notice' ) );
}